\name{mirbase_dbconn}

\alias{mirbase_dbconn}
\alias{mirbase_dbfile}
\alias{mirbase_dbschema}
\alias{mirbase_dbInfo}


\title{Collect information about the package annotation DB}

\description{
  Some convenience functions for getting a connection object to (or collecting
  information about) the package annotation DB.
}

\usage{
  mirbase_dbconn()
  mirbase_dbfile()
  mirbase_dbschema(file="", show.indices=FALSE)
  mirbase_dbInfo()
}

\arguments{
  \item{file}{
    A connection, or a character string naming the file to print to (see
    the \code{file} argument of the \code{\link[base]{cat}} function for
    the details).
  }
  \item{show.indices}{
    The CREATE INDEX statements are not shown by default.
    Use \code{show.indices=TRUE} to get them.
  }
}

\details{
  \code{mirbase_dbconn} returns a connection object to the
  package annotation DB.  IMPORTANT: Don't call
  \code{\link[DBI:dbConnect]{dbDisconnect}} on the connection object
  returned by \code{mirbase_dbconn} or you will break all the
  \code{\link[AnnotationDbi:AnnDbObj-class]{AnnDbObj}} objects defined
  in this package!

  \code{mirbase_dbfile} returns the path (character string) to the
  package annotation DB (this is an SQLite file).

  \code{mirbase_dbschema} prints the schema definition of the
  package annotation DB.

  \code{mirbase_dbInfo} prints other information about the package
  annotation DB.
}

\value{
  \code{mirbase_dbconn}: a DBIConnection object representing an
  open connection to the package annotation DB.

  \code{mirbase_dbfile}: a character string with the path to the
  package annotation DB.

  \code{mirbase_dbschema}: none (invisible \code{NULL}).

  \code{mirbase_dbInfo}: none (invisible \code{NULL}).
}

\seealso{
  \code{\link[DBI:dbSendQuery]{dbGetQuery}},
  \code{\link[DBI:dbConnect]{dbConnect}},
  \code{\link[AnnotationDbi:AnnDbObj-class]{dbconn}},
  \code{\link[AnnotationDbi:AnnDbObj-class]{dbfile}},
  \code{\link[AnnotationDbi:AnnDbObj-class]{dbschema}},
  \code{\link[AnnotationDbi:AnnDbObj-class]{dbInfo}}
}

\examples{
  ## Count the number of rows in the "ID2ACC" table:
  dbGetQuery(mirbase_dbconn(), "SELECT COUNT(*) FROM mirna")

  ## The connection object returned by mirbase_dbconn() was
  ## created with:
  dbConnect(SQLite(), dbname=mirbase_dbfile(), cache_size=64000,
  synchronous=0)

  mirbase_dbschema()

  mirbase_dbInfo()
}

\keyword{utilities}
\keyword{datasets}

